pro port_mask,sme_source,sme_target,TRIAL=trial,DEFAULT_MASK=def_mask
;
; This rountine takes the mask from the source SME structure and interpolates
; it to the target structure. according to the radial velocities.
; History:
;  25-03-2013: NP wrote.
;

  if(n_params() lt 2) then begin
    print,'Usage: port_mask,sme_source,sme_target[,/TRIAL[,DEFAULT=default_mask]]'
    print,' where: sme_source - input SME structure or a save-file with one;'
    print,'        sme_target - output SME structure or a save-file with one;'
    print,'        /TRIAL - dry run with verification graphics. No modifications to the output;'
    print,'        def_mask - default value for the mask. Can be 0 or 1. If this parameter'
    print,'                   is not set the default mask is taken from the output structure.'
    return
  endif

; Input structure
  if(size(sme_source, /TNAME) eq 'STRING') then begin ; We probably have a filename
    if(file_test(sme_source)) then begin              ; Indeed, we have a file
      restore,sme_source
      sme_in=sme
      sme=0
    endif else begin
      print,'PORT_MASK: Input filename "'+sme_source+'" does not exist'
      return
    endelse
  endif else if(size(sme_source, /TNAME) eq 'STRUCT') then begin ; We probably have a structure
    sme_in=sme_source
  endif

; Output structure
  is_file=0B
  if(size(sme_target, /TNAME) eq 'STRING') then begin ; We probably have a filename
    if(file_test(sme_target)) then begin              ; Indeed, we have a file
      restore,sme_target
      sme_out=sme
      sme=0
      is_file=1B
    endif else begin
      print,'PORT_MASK: Output filename "'+sme_target+'" does not exist'
      return
    endelse
  endif else if(size(sme_target, /TNAME) eq 'STRUCT') then begin ; We probably have a structure
    sme_out=sme_target
  endif

; Locate different mask fields in the input structure  
  nobs=n_elements(sme_in.wave)
  ind=where(sme_in.mob[1:*] - sme_in.mob[0:nobs-2L] ne 0, ninterv)
  ind=[ind, nobs-1L]
  if(ninterv gt 0) then ind=[-1L, ind]
  ninterv=n_elements(ind)-1L
  val=bytarr(ninterv)
  for i=0l,ninterv-1L do val[i]=sme_in.mob[ind[i]+1L]

; Create lab wavelength scales for both structures
  clight=2.9979246d5
  wl_in=sme_in.wave
  if(sme_in.vrad_flag eq 0) then begin
     i2=-1L
     for iseg=0,sme_in.nseg-1 do begin
       i1=i2+1L
       i2=sme_in.wind[iseg]
       wl_in[i1:i2]=wl_in[i1:i2]*(1.d0-sme_in.vrad[iseg]/clight)
     endfor
  endif else wl_in=wl_in*(1.d0-sme_in.vrad/clight)

  wl_out=sme_out.wave
  if(sme_out.vrad_flag eq 0) then begin
     i2=-1L
     for iseg=0,sme_out.nseg-1 do begin
       i1=i2+1L
       i2=sme_out.wind[iseg]
       wl_out[i1:i2]=wl_out[i1:i2]*(1.d0-sme_out.vrad[iseg]/clight)
     endfor
  endif else wl_out=wl_out*(1.d0-sme_out.vrad/clight)

; Create new mask
  mask=sme_out.mob
  if(n_elements(def_mask) gt 0) then begin      ; Set to user default if given
    if(def_mask eq 0 or def_mask eq 1) then mask[*]=def_mask
  endif

  for i=0l,ninterv-1 do begin
    ii=where(wl_out ge wl_in[ind[i]+1L] and wl_out le wl_in[ind[i+1L]], nii)
    if(nii gt 0) then mask[ii]=val[i]
  endfor

  if(keyword_set(trial)) then begin
    !p.multi=[0,1,2]
     i2=-1L
     for iseg=0,sme_out.nseg-1 do begin
       i1=i2+1L
       i2=sme_out.wind[iseg]
       xr=[sme_out.wave[i1],sme_out.wave[i2]]
       plot,sme_in.wave,sme_in.sob,tit='Input structure',xr=xr,xs=3,yr=[0,2]
       oplot,sme_in.wave,sme_in.mob,col=c24(2),thick=3,psym=10
       plot,sme_out.wave[i1:i2],sme_out.sob[i1:i2],tit='Output structure. ' $
             +'Segment #'+strtrim(iseg,2),xr=xr,xs=3,yr=[0,2]
       oplot,sme_out.wave[i1:i2],mask[i1:i2],col=c24(2),thick=3,psym=10
       if(iseg lt sme_out.nseg-1) then answ=get_kbrd(1)
     endfor
     !p.multi=0
  endif else begin
    sme_out.mob=mask
    if(is_file) then begin
      sme=sme_out
      save,sme,file=sme_target
    endif else begin
      sme_target=sme_out
    endelse
  endelse

  return
end

;restore,'c:\Users\nikolai\SME-SVN\tools\kolya3_19.03.out'
;sme_in=sme
;restore,'c:\Users\nikolai\SME-SVN\tools\test9_16.03.out'
;sme_out=sme
;
;xr=[4485,4500]
;port_mask,sme_in,sme_out,xr=xr
;
;!p.multi=[0,1,2]
;plot,sme_in.wave,sme_in.sob,tit='Input structure',xr=xr,xs=3,yr=[0,2]
;oplot,sme_in.wave,sme_in.mob,col=c24(2),thick=3,psym=10
;plot,sme_out.wave,sme_out.sob,tit='Output structure',xr=xr,xs=3,yr=[0,2]
;oplot,sme_out.wave,sme_out.mob,col=c24(2),thick=3,psym=10
;end
